@@ -12,36 +12,33 @@ from errno import ENOENT
 import xml.etree.ElementTree as ET
 
 from dwca.files import CSVDataFile
-from dwca.descriptors import ArchiveDescriptor, DataFileDescriptor
-from dwca.exceptions import RowNotFound, InvalidArchive, InvalidSimpleArchive
-
-DEFAULT_METADATA_FILENAME = "EML.xml"
-METAFILE_NAME = "meta.xml"
-SOURCE_METADATA_DIRECTORY = 'dataset'
+from dwca.descriptors import ArchiveDescriptor, DataFileDescriptor, shorten_term
+from dwca.exceptions import RowNotFound, InvalidArchive, InvalidSimpleArchive, NotADataFile
+import dwca.vendor
 
 
 class DwCAReader(object):
     """This class is used to represent a Darwin Core Archive as a whole.
 
     It gives read access to the contained data, to the scientific metadata, ... It supports
-    archives with or without Metafiles, such as described on page 2 of the Reference Guide
-    to the XML Descriptor (http://www.gbif.org/resource/80639).
+    archives with or without Metafile, such as described on page 2 of the `Reference Guide
+    to the XML Descriptor <http://www.gbif.jp/v2/pdf/gbif_dwc-a_metafile_en_v1.pdf>`_.
 
     :param path: path to the Darwin Core Archive (either a zip/tgz file or a directory) to open.
     :type path: str
-    :param extensions_to_ignore: relative path (within the archive) of extension data files to \
-    ignore. This will improve performances and memory consumption with large archives. Missing \
-    files are silently ignored.
+    :param extensions_to_ignore: path (relative to the archive root) of extension data files to ignore. This will \
+    improve speed and memory usage for large archives. Missing files are silently ignored.
     :type extensions_to_ignore: list
 
     :raises: :class:`dwca.exceptions.InvalidArchive`
     :raises: :class:`dwca.exceptions.InvalidSimpleArchive`
 
-    A simple usage example::
+    Usage::
 
         from dwca.read import DwCAReader
 
         dwca = DwCAReader('my_archive.zip')
+
         # Iterating on core rows is easy:
         for core_row in dwca:
             # core_row is an instance of dwca.rows.CoreRow
@@ -53,7 +50,7 @@ class DwCAReader(object):
         # Close the archive to free resources
         dwca.close()
 
-    The archive can also be opened with the `with` statement. This is recommended, since it ensures
+    The archive can also be opened using the `with` statement. This is recommended, since it ensures
     resources will be properly cleaned after usage:
 
     ::
@@ -67,6 +64,10 @@ class DwCAReader(object):
 
     """
 
+    default_metadata_filename  = "EML.xml"
+    default_metafile_name = "meta.xml"
+    source_metadata_directory = 'dataset'
+
     def __enter__(self):
         return self
 
@@ -90,26 +91,31 @@ class DwCAReader(object):
         #: An :class:`descriptors.ArchiveDescriptor` instance giving access to the archive
         #: descriptor/metafile (``meta.xml``)
         try:
-            self.descriptor = ArchiveDescriptor(self.open_included_file(METAFILE_NAME).read(),
+            self.descriptor = ArchiveDescriptor(self.open_included_file(self.default_metafile_name).read(),
                                                 files_to_ignore=extensions_to_ignore)
         except IOError as exc:
             if exc.errno == ENOENT:
                 self.descriptor = None
 
         #: A :class:`xml.etree.ElementTree.Element` instance containing the (scientific) metadata
-        #: of the archive, or None if the Archive contains no metadata.
+        #: of the archive, or `None` if the archive has no metadata.
         self.metadata = self._parse_metadata_file()
-        #: If the archive contains source metadata (typically, GBIF downloads) this dict will
-        #: be something like:
-        #: {'dataset1_UUID': <dataset1 EML (xml.etree.ElementTree.Element instance)>,
-        #: 'dataset2_UUID': <dataset2 EML (xml.etree.ElementTree.Element instance)>, ...}
-        #: see :doc:`gbif_results` for more details.
-        self.source_metadata = self._load_source_metadata()
 
-        if self.descriptor:
-            #  We have an Archive descriptor that we can use to access data files.
-            self._corefile = CSVDataFile(self._workin_directory_path, self.descriptor.core)
-            self._extensionfiles = [CSVDataFile(work_directory=self._workin_directory_path,
+        #: If the archive contains source-level metadata (typically, GBIF downloads), this is a dict such as::
+        #:
+        #:      {'dataset1_UUID': <dataset1 EML> (xml.etree.ElementTree.Element object),
+        #:       'dataset2_UUID': <dataset2 EML> (xml.etree.ElementTree.Element object), ...}
+        #:
+        #: See :doc:`gbif_results` for more details.
+        self.source_metadata = self._get_source_metadata()
+
+        if self.descriptor:  # We have an Archive descriptor that we can use to access data files.
+            #: An instance of :class:`dwca.files.CSVDataFile` for the core data file.
+            self.core_file = CSVDataFile(self._workin_directory_path, self.descriptor.core)
+
+            #: A list of :class:`dwca.files.CSVDataFile`, one entry for each extension data file , sorted by order of
+            #: appearance in the Metafile (or an empty list if the archive doesn't use extensions).
+            self.extension_files = [CSVDataFile(work_directory=self._workin_directory_path,
                                                 file_descriptor=d)
                                     for d in self.descriptor.extensions]
         else:  # Archive without descriptor, we'll have to find and inspect the data file
@@ -117,23 +123,75 @@ class DwCAReader(object):
                 datafile_name = self._is_valid_simple_archive()
                 descriptor = DataFileDescriptor.make_from_file(os.path.join(self._workin_directory_path, datafile_name))
 
-                self._corefile = CSVDataFile(work_directory=self._workin_directory_path,
+                self.core_file = CSVDataFile(work_directory=self._workin_directory_path,
                                              file_descriptor=descriptor)
-                self._extensionfiles = []
+                self.extension_files = []
             except InvalidSimpleArchive:
-                msg = "No metafile was found, but archive includes multiple files/directories."
+                msg = "No Metafile was found, but the archive contains multiple files/directories."
                 raise InvalidSimpleArchive(msg)
 
-    def _load_source_metadata(self):
-        r = {}
+    def _get_source_metadata(self):
+        source_metadata = {}
+        source_metadata_dir = os.path.join(self._workin_directory_path, self.source_metadata_directory)
 
-        dataset_dir = os.path.join(self._workin_directory_path, SOURCE_METADATA_DIRECTORY)
-        if os.path.isdir(dataset_dir):
-            for f in os.listdir(dataset_dir):
-                if os.path.isfile(os.path.join(dataset_dir, f)):
-                    key = os.path.splitext(f)[0]
-                    r[key] = self._parse_xml_included_file(os.path.join(SOURCE_METADATA_DIRECTORY, f))
-        return r
+        if os.path.isdir(source_metadata_dir):
+            for f in os.listdir(source_metadata_dir):
+                if os.path.isfile(os.path.join(source_metadata_dir, f)):
+                    dataset_key = os.path.splitext(f)[0]
+                    source_metadata[dataset_key] = self._parse_xml_included_file(
+                        os.path.join(self.source_metadata_directory, f))
+
+        return source_metadata
+
+    def pd_read(self, relative_path, **kwargs):
+        """Return a `Pandas <https://pandas.pydata.org>`_ \
+        `DataFrame <https://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.html>`_ for the data file \
+        located at `relative_path`.
+
+        This method wraps pandas.read_csv() and accept the same keyword arguments. The following arguments will be
+        ignored (because they are set appropriately for the data file): `delimiter`, `skiprows`, `header` and `names`.
+
+        :param relative_path: path to the data file (relative to the archive root).
+        :type relative_path: str
+
+        :raises: `ImportError` if Pandas is not installed.
+        :raises: :class:`dwca.exceptions.NotADataFile` if `relative_path` doesn't designate a valid data file\
+        in the archive.
+
+        .. warning::
+
+            You'll need to `install Pandas <http://pandas.pydata.org/pandas-docs/stable/install.html>`_ before using
+            this method.
+
+        .. note::
+
+            Default values of Darwin Core Archive are supported: A column will be added to the DataFrame if a term has
+            a default value in the Metafile (but no corresponding column in the CSV Data File).
+        """
+        datafile_descriptor = self.get_descriptor_for(relative_path)
+
+        if datafile_descriptor is None:
+            raise NotADataFile()
+
+        if not dwca.vendor._has_pandas:
+            raise ImportError("Pandas is missing.")
+
+        from pandas import read_csv
+
+        kwargs['delimiter'] = datafile_descriptor.fields_terminated_by
+        kwargs['skiprows'] = datafile_descriptor.lines_to_ignore
+        kwargs['header'] = None
+        kwargs['names'] = datafile_descriptor.short_headers
+
+        df = read_csv(self.absolute_temporary_path(relative_path), **kwargs)
+
+        # Add a column for default values, if present in the file descriptor
+        for field in datafile_descriptor.fields:
+            field_default_value = field['default']
+            if field_default_value is not None:
+                df[shorten_term(field['term'])] = field_default_value
+
+        return df
 
     def orphaned_extension_rows(self):
         """Return a dict of the orphaned extension rows.
@@ -144,48 +202,51 @@ class DwCAReader(object):
          {'description.txt': {u'5': [3, 4], u'6': [5]},
           'vernacularname.txt': {u'7': [4]}}
 
-        (meaning: in description.txt, rows at position 3 and 4 reference a core row whose ID is '5', but such a core
-        row doesn't exists).
+        Meaning:
+
+            * in `description.txt`, rows at position 3 and 4 reference a core row whose ID is '5', but such a core \
+            row doesn't exists. Row at position 5 references an imaginary core row with ID '6'
+            * in `vernacularname.txt`, the row at position 4 references an imaginary core row with ID '7'
 
         """
-        if len(self._extensionfiles) > 0:
+        indexes = {}
 
+        if len(self.extension_files) > 0:
             temp_ids = {}
             for row in self:
                 temp_ids[row.id] = 1
             ids = temp_ids.keys()
 
-            indexes = {}
-            for extension in self._extensionfiles:
+            for extension in self.extension_files:
                 coreid_index = extension.coreid_index.copy()
                 for id in ids:
                     coreid_index.pop(id, None)
                 indexes[extension.file_descriptor.file_location] = coreid_index
 
-            return indexes
-
-        else:
-            return {}
+        return indexes
 
     @property
     def use_extensions(self):
-        """Return `True` if the Archive makes use of extensions."""
+        """`True` if the archive makes use of extensions."""
         return (self.descriptor is not None) and self.descriptor.extensions
 
     @property
     # TODO: decide, test and document what we guarantee about ordering
     def rows(self):
-        """A list of :class:`rows.CoreRow` instances representing the content of the archive.
+        """A list of :class:`rows.CoreRow` objects representing the content of the archive.
 
         .. warning::
 
-            This will cause all rows to be loaded in memory. In case of large Darwin Core Archive,
-            you may prefer iterating with a for loop.
+            All rows will be loaded in memory. In case of a large Darwin Core Archive, you may prefer iterating with
+            a for loop.
         """
         return list(self)
 
     def get_corerow_by_id(self, row_id):
-        """Return the (core) row whose ID is row_id.
+        """Return the (core) row whose ID is `row_id`.
+
+        :param row_id: ID of the core row you want
+        :type row_id: str
 
         :returns:  :class:`dwca.rows.CoreRow` -- the matching row.
         :raises: :class:`dwca.exceptions.RowNotFound`
@@ -206,8 +267,7 @@ class DwCAReader(object):
         raise RowNotFound
 
     def get_row_by_id(self, row_id):
-        """Return the (core) row whose id is row_id.
-
+        """
         .. warning::
 
             Deprecated: this method has been renamed to :meth:`get_corerow_by_id`.
@@ -217,28 +277,29 @@ class DwCAReader(object):
         warnings.warn("This method has been renamed to get_corerow_by_id().", DeprecationWarning)
         return self.get_corerow_by_id(row_id)
 
-    def get_corerow_by_position(self, index):
+    def get_corerow_by_position(self, position):
         """Return a core row according to its position/index in core file.
 
+        :param position: the position (starting at 0) of the row you want in the core file.
+        :type position: int
+
         :returns:  :class:`dwca.rows.CoreRow` -- the matching row.
         :raises: :class:`dwca.exceptions.RowNotFound`
 
         .. note::
 
-            - First row has index 0
             - If index is bigger than the length of the archive, None is returned
             - The position is often an appropriate way to unambiguously identify a core row in a DwCA.
 
         """
         for (i, row) in enumerate(self):
-            if i == index:
+            if i == position:
                 return row
 
         raise RowNotFound
 
     def get_row_by_index(self, index):
-        """Return a core row according to its position/index in core file.
-
+        """
         .. warning::
 
             Deprecated: this method has been renamed to :meth:`get_corerow_by_position`.
@@ -249,15 +310,26 @@ class DwCAReader(object):
         return self.get_corerow_by_position(index)
 
     def absolute_temporary_path(self, relative_path):
-        """Return the absolute path of the file located at relative_path within the archive.
+        """Return the absolute path of a file located within the archive.
+
+        This method allows raw access to the files contained in the archive. It can be useful to open additional, \
+        non-standard files embedded in the archive, or to open a standard file with another library.
+
+        :param relative_path: the path (relative to the archive root) of the file.
+        :type relative_path: str
+
+        :returns: the absolute path to the file.
+
+        Usage::
+
+            dwca.absolute_temporary_path('occurrence.txt')  # => /tmp/afdfsec7/occurrence.txt
+
+        .. warning::
+            If the archive is contained in a zip or tgz file, the returned path will point to a temporary file that \
+            will be removed when closing the :class:`dwca.read.DwCAReader` instance.
 
         .. note::
-            - This method allows raw access to the files contained in the archive. It is for\
-            example useful to open additional, non-standard files embedded in the archive.
-            - If the archive is contained in a zip or tgz file, the returned path will point to a\
-            temporary file that will be removed when closing the :class:`dwca.read.DwCAReader`\
-            instance.
-            - File existence is not tested.
+            File existence is not tested.
 
         """
         return os.path.abspath(os.path.join(self._workin_directory_path, relative_path))
@@ -268,21 +340,22 @@ class DwCAReader(object):
         :param relative_path: the path (relative to the archive root) to the data file you want info about.
         :type relative_path: str
 
-        :returns:  :class:`dwca.descriptors.DataFileDescriptor` -- the descriptor or `None` if `relative_path` doesn't \
-        reference a valid data file.
+        :returns:  :class:`dwca.descriptors.DataFileDescriptor`
+        :raises: :class:`dwca.exceptions.NotADataFile` if `relative_path` doesn't reference a valid data file.
 
         Examples::
 
             dwca.get_descriptor_for('occurrence.txt')
             dwca.get_descriptor_for('verbatim.txt')
         """
-
-        all_datafiles = [self._corefile] + self._extensionfiles
+        all_datafiles = [self.core_file] + self.extension_files
 
         for datafile in all_datafiles:
             if datafile.file_descriptor.file_location == relative_path:
                 return datafile.file_descriptor
 
+        raise NotADataFile("{fn} is not a data file".format(fn=relative_path))
+
     def _is_valid_simple_archive(self):
         # If the working dir appear to contains a valid simple darwin core archive
         # (one single data file + possibly some metadata), returns the name of the data file.
@@ -294,22 +367,25 @@ class DwCAReader(object):
             return files[0]
         elif len(files) == 2:
             # Two files found: if one of them is EML.xml, the other is considered as the data file
-            if DEFAULT_METADATA_FILENAME in files:
-                return [f for f in files if f is not DEFAULT_METADATA_FILENAME][0]
+            if self.default_metadata_filename in files:
+                return [f for f in files if f is not self.default_metadata_filename][0]
 
         raise InvalidSimpleArchive()
 
-    # TODO: Document: Win won't be able to cleanup if some files are not closed
     def open_included_file(self, relative_path, *args, **kwargs):
-        """Simple wrapper around Python's open build-in function.
+        """Simple wrapper around Python's build-in `open` function.
 
-        To be used for reading only.
+        To be used only for reading.
+
+        .. warning ::
+            Don't forget to close the files after usage. This is especially important on Windows because temporary
+            (extracted) files won't be cleanable if not closed.
         """
         return open(self.absolute_temporary_path(relative_path), *args, **kwargs)
 
     def _parse_metadata_file(self):
         """Load the archive (scientific) Metadata file, parse it with\
-        ElementTree and return its content (or None if the Archive contains no metadata).
+        ElementTree and return its content (or `None` if the archive has no metadata).
 
         :raises: :class:`dwca.exceptions.InvalidArchive` if the archive references an inexisting
         metadata file.
@@ -327,14 +403,14 @@ class DwCAReader(object):
 
         else:  # Otherwise, the metadata file has to be named 'EML.xml'
             try:
-                return self._parse_xml_included_file(DEFAULT_METADATA_FILENAME)
+                return self._parse_xml_included_file(self.default_metadata_filename)
             except IOError as e:
                 if e.errno == ENOENT:  # File not found, this is an archive without metadata
                     return None
 
     def _parse_xml_included_file(self, relative_path):
         """Load, parse and returns (as ElementTree.Element) XML file located at relative_path."""
-        return ET.fromstring(self.open_included_file(relative_path).read())
+        return ET.parse(self.absolute_temporary_path(relative_path)).getroot()
 
     def _unzip_or_untar(self):
         """Create a temporary dir. and uncompress/unarchive self.archive_path there.
@@ -386,8 +462,8 @@ class DwCAReader(object):
 
         """
         #  Windows can't remove a dir with opened files
-        self._corefile.close()
-        for extension_file in self._extensionfiles:
+        self.core_file.close()
+        for extension_file in self.extension_files:
             extension_file.close()
 
         if self._directory_to_clean:
@@ -397,8 +473,8 @@ class DwCAReader(object):
         rmtree(self._directory_to_clean, False)
 
     def core_contains_term(self, term_url):
-        """Return True if the Core file of the archive contains the term_url term."""
-        return term_url in self._corefile.file_descriptor.terms
+        """Return `True` if the Core file of the archive contains the `term_url` term."""
+        return term_url in self.core_file.file_descriptor.terms
 
     def __iter__(self):
         self._corefile_pointer = 0
@@ -408,10 +484,10 @@ class DwCAReader(object):
         return self.next()
 
     def next(self):  # NOQA
-        r = self._corefile.get_row_by_position(self._corefile_pointer)
+        r = self.core_file.get_row_by_position(self._corefile_pointer)
         if r:
             # Set up linked data so the CoreRow will know about them
-            r.link_extension_files(self._extensionfiles)
+            r.link_extension_files(self.extension_files)
             r.link_source_metadata(self.source_metadata)
 
             self._corefile_pointer = self._corefile_pointer + 1
@@ -421,20 +497,20 @@ class DwCAReader(object):
 
 
 class GBIFResultsReader(DwCAReader):
-    """This class is used to represent the slightly augmented variant of Darwin Core Archive\
-    produced by the new GBIF Data Portal when exporting occurrences.
+    """This class is used to represent the slightly augmented variant of Darwin Core Archive produced by the GBIF Data
+    Portal when exporting occurrences.
 
-    .. warning:: This class is deprecated. See :doc:`gbif_results` to learn how to achive the same\
-    results with :class:`.DwCAReader`.
+    .. warning:: This class is deprecated. See :doc:`gbif_results` to learn how to achieve the same results with \
+    :class:`.DwCAReader`.
 
     """
 
     @property
     def citations(self):
-        """Return the content of the citations.txt file included in the archive."""
+        """The content of the `citations.txt` file included in the archive."""
         return self.open_included_file('citations.txt').read()
 
     @property
     def rights(self):
-        """Return the content of the rights.txt file included in the archive."""
+        """The content of the `rights.txt` file included in the archive."""
         return self.open_included_file('rights.txt').read()
