@@ -8,10 +8,15 @@ import warnings
 
 import xml.etree.ElementTree as ET
 
+from mock import patch
+
+import pandas as pd
+
+from dwca.files import CSVDataFile
 from dwca.read import DwCAReader, GBIFResultsReader
 from dwca.rows import CoreRow, ExtensionRow
 from dwca.darwincore.utils import qualname as qn
-from dwca.exceptions import RowNotFound, InvalidArchive
+from dwca.exceptions import RowNotFound, InvalidArchive, NotADataFile
 from dwca.descriptors import ArchiveDescriptor, DataFileDescriptor
 
 from .helpers import (GBIF_RESULTS_PATH, BASIC_ARCHIVE_PATH, EXTENSION_ARCHIVE_PATH,
@@ -22,12 +27,117 @@ from .helpers import (GBIF_RESULTS_PATH, BASIC_ARCHIVE_PATH, EXTENSION_ARCHIVE_P
                       SIMPLE_CSV_DOS, BASIC_ENCLOSED_ARCHIVE_PATH, INVALID_SIMPLE_TOOMUCH,
                       INVALID_SIMPLE_TWO, SIMPLE_CSV_NOTENCLOSED, NOMETADATA_PATH,
                       DEFAULT_METADATA_FILENAME, BASIC_ARCHIVE_TGZ_PATH, INVALID_DESCRIPTOR,
-                      DWCA_ORPHANED_ROWS)
+                      DWCA_ORPHANED_ROWS, METADATA_1252_ENCODING)
 
 
+class TestPandasIntegration(unittest.TestCase):
+    """Tests of Pandas integration features."""
+    # TODO: test weirder archives (encoding, lime termination, ...)
+
+    @patch('dwca.vendor._has_pandas', False)
+    def test_pd_read_pandas_unavailable(self):
+        with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
+            with self.assertRaises(ImportError):
+                dwca.pd_read('occurrence.txt')
+
+    def test_pd_read_simple_case(self):
+        with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
+            df = dwca.pd_read('occurrence.txt')
+
+            # check types, headers and dimensions
+            self.assertIsInstance(df, pd.DataFrame)
+            cols = df.columns.values.tolist()
+            self.assertEqual(cols, ['id', 'basisOfRecord', 'locality', 'family', 'scientificName'])
+            self.assertEqual(df.shape, (2, 5))  # Row/col counts are correct
+
+            # check content
+            self.assertEqual(df['basisOfRecord'].values.tolist(), ['Observation', 'Observation'])
+            self.assertEqual(df['family'].values.tolist(), ['Tetraodontidae', 'Osphronemidae'])
+            self.assertEqual(df['locality'].values.tolist(), ['Borneo', 'Mumbai'])
+            self.assertEqual(df['scientificName'].values.tolist(), ['tetraodon fluviatilis', 'betta splendens'])
+
+    def test_pd_read_no_data_files(self):
+        with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
+            with self.assertRaises(NotADataFile):
+                dwca.pd_read('imaginary_file.txt')
+
+            with self.assertRaises(NotADataFile):
+                dwca.pd_read('eml.xml')
+
+    def test_pd_read_extensions(self):
+        with DwCAReader(MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
+            desc_df = dwca.pd_read('description.txt')
+            self.assertIsInstance(desc_df, pd.DataFrame)
+            self.assertEqual(desc_df.shape, (3, 4))
+            self.assertEqual(desc_df['language'].values.tolist(), ['EN', 'FR', 'EN'])
+
+            vern_df = dwca.pd_read('vernacularname.txt')
+            self.assertIsInstance(vern_df, pd.DataFrame)
+            self.assertEqual(vern_df.shape, (4, 4))
+            self.assertEqual(vern_df['countryCode'].values.tolist(), ['US', 'ZA', 'FI', 'ZA'])
+
+    def test_pd_read_quotedir(self):
+        with DwCAReader(DIRECTORY_CSV_QUOTE_ARCHIVE_PATH) as dwca:
+            df = dwca.pd_read('occurrence.txt')
+            # The field separator is found in a quoted field, don't break
+            self.assertEqual(df.shape, (2, 5))
+            self.assertEqual(df['basisOfRecord'].values.tolist()[0], 'Observation, something')
+
+    def test_pd_read_default_values(self):
+        with DwCAReader(DEFAULT_VAL_PATH) as dwca:
+            df = dwca.pd_read('occurrence.txt')
+
+            self.assertIn('country', df.columns.values.tolist())
+            for country in df['country'].values.tolist():
+                self.assertEqual(country, 'Belgium')
+
+    def test_pd_read_utf8_eol_ignored(self):
+        """Ensure we don't split lines based on the x85 utf8 EOL char.
+
+        (only the EOL string specified in meta.xml should be used).
+         """
+        with DwCAReader(UTF8EOL_ARCHIVE_PATH) as dwca:
+            df = dwca.pd_read('occurrence.txt')
+            # If line properly split => 64 columns.
+            # (61 - and probably an IndexError - if errors)
+            self.assertEqual(64, df.shape[1])
+
+    def test_pd_read_simple_csv(self):
+        with DwCAReader(SIMPLE_CSV) as dwca:
+
+            df = dwca.pd_read('0008333-160118175350007.csv')
+            # Ensure we get the correct number of rows
+            self.assertEqual(3, df.shape[0])
+            # Ensure we can access arbitrary data
+
+            self.assertEqual(df['decimallatitude'].values.tolist()[1], -31.98333)
+
 class TestDwCAReader(unittest.TestCase):
     # TODO: Move row-oriented tests to another test class
     """Unit tests for DwCAReader class."""
+    def test_core_file(self):
+        with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
+            self.assertIsInstance(dwca.core_file, CSVDataFile)
+
+            # Quick content check just to be sure
+            self.assertEqual(dwca.core_file.lines_to_ignore, 1)
+
+    def test_extension_file_noext(self):
+        with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
+            self.assertEqual(dwca.extension_files, [])
+
+    def test_extension_files(self):
+        with DwCAReader(MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
+            # Check extension_files is iterable and contains the right type
+            for ext in dwca.extension_files:
+                self.assertIsInstance(ext, CSVDataFile)
+
+            # Check the length is correct
+            self.assertEqual(len(dwca.extension_files), 2)
+
+            # Check the order of the metafile is respected + quick content check
+            self.assertEqual(dwca.extension_files[0].file_descriptor.file_location, 'description.txt')
+            self.assertEqual(dwca.extension_files[1].file_descriptor.file_location, 'vernacularname.txt')
 
     def test_get_descriptor_for(self):
         with DwCAReader(MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
@@ -36,12 +146,16 @@ class TestDwCAReader(unittest.TestCase):
             self.assertIsInstance(dwca.get_descriptor_for('description.txt'), DataFileDescriptor)
             self.assertIsInstance(dwca.get_descriptor_for('vernacularname.txt'), DataFileDescriptor)
 
-            # But None for non-data files
-            self.assertIsNone(dwca.get_descriptor_for('eml.xml'))
-            self.assertIsNone(dwca.get_descriptor_for('meta.xml'))
+            # But NotADataFile exception for non-data files
+            with self.assertRaises(NotADataFile):
+                dwca.get_descriptor_for('eml.xml')
 
-            # Also None for files that don't actually exists
-            self.assertIsNone(dwca.get_descriptor_for('imaginary_file.txt'))
+            with self.assertRaises(NotADataFile):
+                dwca.get_descriptor_for('meta.xml')
+
+            # Also NotADataFile for files that don't actually exists
+            with self.assertRaises(NotADataFile):
+                dwca.get_descriptor_for('imaginary_file.txt')
 
             # Basic content checks of the descriptors
             taxon_descriptor = dwca.get_descriptor_for('taxon.txt')
@@ -60,7 +174,7 @@ class TestDwCAReader(unittest.TestCase):
             self.assertEqual(vernacular_descriptor.file_encoding, 'utf-8')
             self.assertEqual(vernacular_descriptor.type, 'http://rs.gbif.org/terms/1.0/VernacularName')
 
-        # Also check we can get a DataFileDescriptor with a simple Archive (no metafile)
+        # Also check we can get a DataFileDescriptor for a simple Archive (without metafile)
         with DwCAReader(SIMPLE_CSV) as dwca:
             self.assertIsInstance(dwca.get_descriptor_for('0008333-160118175350007.csv'), DataFileDescriptor)
 
@@ -84,7 +198,7 @@ class TestDwCAReader(unittest.TestCase):
         """
         with DwCAReader(INVALID_DESCRIPTOR) as dwca:
             with self.assertRaises(InvalidArchive):
-                for row in dwca:
+                for _ in dwca:
                     pass
 
     def test_use_extensions(self):
@@ -113,8 +227,10 @@ class TestDwCAReader(unittest.TestCase):
         with DwCAReader(DEFAULT_METADATA_FILENAME) as dwca:
             self.assertIsInstance(dwca.metadata, ET.Element)
 
-            v = (dwca.metadata.find('dataset').find('creator').find('individualName')
-                     .find('givenName').text)
+            v = (dwca.metadata.find('dataset')
+                              .find('creator')
+                              .find('individualName')
+                              .find('givenName').text)
             self.assertEqual(v, 'Nicolas')
 
     def test_subdirectory_archive(self):
@@ -122,7 +238,6 @@ class TestDwCAReader(unittest.TestCase):
         tmp_dir = tempfile.gettempdir()
 
         num_files_before = len(os.listdir(tmp_dir))
-        num_files_during = None
         with DwCAReader(SUBDIR_ARCHIVE_PATH) as dwca:
             # Ensure we have access to metadata
             self.assertIsInstance(dwca.metadata, ET.Element)
@@ -143,7 +258,7 @@ class TestDwCAReader(unittest.TestCase):
         self.assertEqual(num_files_before, num_files_after)
 
     def test_exception_invalid_archives_missing_metadata(self):
-        """Ensure an exception is raised when referencing a missing metadata file."""
+        """An exception is raised when referencing a missing metadata file."""
         # Sometimes, the archive metafile references a metadata file that's not present in the
         # archive. See for example http://dev.gbif.org/issues/browse/PF-2125
         with self.assertRaises(InvalidArchive) as cm:
@@ -155,6 +270,22 @@ class TestDwCAReader(unittest.TestCase):
         expected_message = "eml.xml is referenced in the archive descriptor but missing."
         self.assertEqual(str(the_exception), expected_message)
 
+    def test_implicit_encoding_metadata(self):
+        """If the metadata file doesn't specifies encoding, use UTF-8."""
+
+        with DwCAReader(DIRECTORY_ARCHIVE_PATH) as dwca:
+            v = (dwca.metadata.find('dataset').find('creator').find('individualName')
+                 .find('surName').text)
+            self.assertEqual(v, u'Noé')
+
+    def test_explicit_encoding_metadata(self):
+        """If the metadata file explicitly specifies encoding (<xml ...>), make sure it is used."""
+
+        with DwCAReader(METADATA_1252_ENCODING) as dwca:
+            v = (dwca.metadata.find('dataset').find('creator').find('individualName')
+                 .find('surName').text)
+            self.assertEqual(v, u'Noé')  # Is the accent properly interpreted?
+
     def test_exception_invalid_simple_archives(self):
         """Ensure an exception is raised when simple archives can't be interpreted.
 
@@ -233,7 +364,6 @@ class TestDwCAReader(unittest.TestCase):
         EML.xml. This correspond to the second case on page #2 of
         http://www.gbif.org/resource/80639. The metadata file having the "standard name", it should
         properly handled.
-
         """
         with DwCAReader(SIMPLE_CSV_EML) as dwca:
             # Ensure we get the correct number of rows
@@ -266,7 +396,7 @@ class TestDwCAReader(unittest.TestCase):
             self.assertEqual(rows[0].data[qn('basisOfRecord')], 'Observation, something')
 
     def test_dont_enclose_unenclosed(self):
-        """If fields_enclosed_by is an empty string, don't enclose (even if quotes are present)"""
+        """If fields_enclosed_by is set to an empty string, don't enclose (even if quotes are present)"""
         with DwCAReader(DIRECTORY_ARCHIVE_PATH) as dwca:
             rows = list(dwca)
 
@@ -399,9 +529,9 @@ class TestDwCAReader(unittest.TestCase):
         self.assertEqual(num_files_before, num_files_after)
 
     def test_source_data_not_destroyed_directory(self):
-        """In archive=directory, it should not be destroyed after use.
+        """If archive is a directory, it should not be deleted after use.
 
-        (check that the cleanup routine for zipped file is not accidentaly called)
+        (check that the cleanup routine for zipped file is not called by accident)
         """
         r = DwCAReader(DIRECTORY_ARCHIVE_PATH)
         r.close()
@@ -414,7 +544,7 @@ class TestDwCAReader(unittest.TestCase):
     def test_temporary_dir_zipped(self):
         """Test a temporary directory is created during execution.
 
-        (complementay to test_cleanup()
+        (complementary to test_cleanup())
         """
         tmp_dir = tempfile.gettempdir()
 
@@ -491,8 +621,8 @@ class TestDwCAReader(unittest.TestCase):
                 self.assertIsInstance(row, CoreRow)
 
     def test_iterate_order(self):
-        """Test that the order of appaearance in Core file is respected when iterating."""
-        # This is also probably tested inderectly elsewhere, but this is the right place :)
+        """Test that the order of appearance in Core file is respected when iterating."""
+        # This is also probably tested indirectly elsewhere, but this is the right place :)
         with DwCAReader(IDS_ARCHIVE_PATH) as dwca:
             l = list(dwca)
             # Row IDs are ordered like this in core file: id 4-1-3-2
@@ -523,7 +653,6 @@ class TestDwCAReader(unittest.TestCase):
                 assert issubclass(the_warning.category, DeprecationWarning)
                 self.assertEqual("This method has been renamed to get_corerow_by_id().", str(the_warning.message))
 
-
     def test_deprecated_row_by_position(self):
         """get_row_by_index() has been renamed get_corerow_by_position(). Make sure it still works, w/ warning."""
 
@@ -778,18 +907,18 @@ class TestDwCAReader(unittest.TestCase):
 
         with DwCAReader(UTF8EOL_ARCHIVE_PATH) as dwca:
             rows = dwca.rows
-            # If line properly splitted => 64 rows.
-            # (61 - and probably an IndexError - if errrors)
+            # If line properly split => 64 columns.
+            # (61 - and probably an IndexError - if errors)
             self.assertEqual(64, len(rows[0].data))
 
     def test_source_metadata(self):
-        # Sandard archive: no source metadata
+        # Standard archive: no source metadata
         with DwCAReader(EXTENSION_ARCHIVE_PATH) as star_dwca:
             self.assertEqual(star_dwca.source_metadata, {})
 
         # GBIF download: source metadata present
         with DwCAReader(GBIF_RESULTS_PATH) as results:
-            # We have 23 EML files in dataset/
+            # We have 23 EML files in the dataset directory
             self.assertEqual(23, len(results.source_metadata))
             # Assert a key is present
             self.assertTrue('eccf4b09-f0c8-462d-a48c-41a7ce36815a' in
@@ -804,8 +933,9 @@ class TestDwCAReader(unittest.TestCase):
             self.assertIsInstance(metadata, ET.Element)
 
             # Assert we can read basic fields from EML:
-            self.assertEqual(metadata.find('dataset').find('creator').find('individualName')
-                             .find('givenName').text,
+            self.assertEqual(metadata.find('dataset')
+                                     .find('creator').find('individualName')
+                                     .find('givenName').text,
                              'Rob')
 
     def test_row_source_metadata(self):
